# Copyright (c) 2021, VRAI Labs and/or its affiliates. All rights reserved.
#
# This software is licensed under the Apache License, Version 2.0 (the
# "License") as published by the Apache Software Foundation.
#
# You may not use this file except in compliance with the License. You may
# obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
SUPPORTED_CDI_VERSIONS = ['2.9']
VERSION = '0.2.1'
TELEMETRY = '/telemetry'
USER_COUNT = '/users/count'
USERS = '/users'
TELEMETRY_SUPERTOKENS_API_URL = 'https://api.supertokens.io/0/st/telemetry'
TELEMETRY_SUPERTOKENS_API_VERSION = '2'
ERROR_MESSAGE_KEY = 'message'
API_KEY_HEADER = 'api-key'
RID_KEY_HEADER = 'rid'
FDI_KEY_HEADER = 'fdi-version'
API_VERSION = '/apiversion'
API_VERSION_HEADER = 'cdi-version'
