# Copyright (c) 2021, VRAI Labs and/or its affiliates. All rights reserved.
#
# This software is licensed under the Apache License, Version 2.0 (the
# "License") as published by the Apache Software Foundation.
#
# You may not use this file except in compliance with the License. You may
# obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
SESSION_REFRESH = '/session/refresh'
SIGNOUT = '/signout'
ACCESS_TOKEN_COOKIE_KEY = 'sAccessToken'
REFRESH_TOKEN_COOKIE_KEY = 'sRefreshToken'
ID_REFRESH_TOKEN_COOKIE_KEY = 'sIdRefreshToken'
FRONT_TOKEN_HEADER_SET_KEY = 'front-token'
ANTI_CSRF_HEADER_KEY = 'anti-csrf'
RID_HEADER_KEY = 'rid'
ID_REFRESH_TOKEN_HEADER_SET_KEY = 'id-refresh-token'
ID_REFRESH_TOKEN_HEADER_GET_KEY = 'id-refresh-token'
ACCESS_CONTROL_EXPOSE_HEADERS = 'Access-Control-Expose-Headers'
